
// Parte del paquete de GP
   package GP;

/**
 * Esta clase representa la estructura de una ADF.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class ADFStru extends Rama {

   /**
    * La informacion de construccion del ADF.<BR> */
      protected TipoADF    tipo;



   /**
    * Un constructor vacio.<BR> */
      public ADFStru() { }

   /**
    * Construye una estructura de ADF de acuerdo a su definicion de tipo.<BR>
    *
    * @param tip
    *    Una instancia con las especificaciones de
    *    construccion para la ADF.<BR> */
      public ADFStru( TipoADF tip ) {
         this.tipo = tip;
         
      // Crea el arbol en forma recursiva
         raiz = nuevaRama( 1, tipo.getProfIni(), tipo.getBalance() );
      }


   /**
    * Construye una estructura de ADF de acuerdo a su definicion de tipo
    * pero con caracteristicas de profundidad y balanceo especiales.<BR>
    *
    * @param tip
    *    Una instancia con las especificaciones de
    *    construccion para la ADF.<BR>
    * @param prf
    *    La profundidad maxima de construccion.<BR>
    * @param bal
    *    El indicador de balanceo en la ADF.<BR> */
      public ADFStru( TipoADF tip, int prf, boolean bal ) {
         this.tipo = tip;
         
      // Crea el arbol en forma recursiva
         raiz = nuevaRama( 1, prf, bal );
      }
 

   /**
    * Produce una copia "profunda" del ADFStru.<BR> */
      public Object clone() {
         ADFStru clon = new ADFStru();

      // OJO: el tipo no se clona, solo se copia la referencia
         clon.tipo = this.tipo;

      // Clona los nodos de la ADF
         clon.raiz = (Funcional) this.raiz.clone();

      // El numero de nodos en la ADF y la profundidad
         clon.prof = this.prof;
         clon.nodos_totales = this.nodos_totales;

         return clon;
      }


   /**
    * Devuelve el nombre de la clase que IMPLEMENTA la ADF.<BR> */
      public String getNombre() { return tipo.getNombre(); }


   /**
    * Crea una nueva rama con caracteristicas de ADFStru.<BR>
    *
    * En este metodo, que sobreescribe al de la clase <B>Rama</B>,
    * se inicializa aleatoriamente el campo <B>narg</B> de los
    * terminales que son de clase <B>ArgADF</B>.<BR>
    *
    * @param p_act
    *    Es el indicador de la profunfidad actual del proceso
    *    en el descenso recursivo.<BR>
    * @param p_max
    *    Es la profundidad maxima con la que se debe construir
    *    la rama.<BR>
    * @param bal
    *    Indica si la rama debe ser o NO balanceada.<BR> */
      private Funcional nuevaRama( int p_act,
                                   int p_max,
                               boolean bal ) {
      
      // Elige el grupo del cual se instancia el atomo
      //   T -> Terminal,   N -> No terminal
      // OJO: En arboles no balanceados echamos un volado !
         char       grupo;
         if ( p_act >= p_max )                       grupo = 'T';
         else if ( p_act == 1 )                      grupo = 'N';
         else if ( bal )                             grupo = 'N';
         else grupo = Random.randomFloat() < 0.5f ?  'T'  :  'N';

      // Crea el atomo
         Funcional nodo = null;
         switch (grupo) {
            case 'T': nodo = tipo.getTerminales().nuevo( 1 );
                      ((ArgADF) nodo).setNArg(
                                         Random.randomInt( tipo.getNArgs() ) );
                      break;
            case 'N': nodo = tipo.getNoTerminales().nuevo(
                              Random.randomInt( tipo.getNoTerminales().n() ) );
                      break;
         }

      // Ahora crea las sub ramas del atomo
         for (int i=1; i<=nodo.getNOp(); i++)
            nodo.setOp( i, nuevaRama( p_act+1, p_max, bal ) );

      // Suma uno al total de nodos en la rama
         nodos_totales++;

      // La profundidad maxima de la rama
         if ( p_act > prof )   prof = p_act;
         
      // Regresa el atomo creado
         return nodo;
      }


   /**
    * Dispara la sustitucion en la estructura del ADF de todos los
    * nodos de la clase <B>ArgADF</B> por los operandos del
    * funcional <B>nodo</B>.<BR>
    *
    * @param sujeto
    *
    * @param nodo
    *    Es el nodo.<BR> */
      public static Funcional expandeADF( Individuo ind,
                                          Funcional fun ) {
      
      // Cual es el nombre de la clase Funcional-ADF ?
         Class c = fun.getClass();
         String nombre_adf = c.getName();

      // Obtiene la referencia del ADFStru del individuo que matcha
      // con el nombre y solicita su expansion con el Funcional
      // actual como fuente.
         Funcional adf_raiz = 
                    (Funcional)
                    ind.getADFStru( nombre_adf ).getRaiz().clone();
         return (Funcional) adf_raiz.expandeADF( fun.operandos );
      }
   }

