
// Parte del paquete de GP
   package GP;

/**
 * La clase <B>AtomLink</B> sirve para identificar ligas
 * de union entre dos nodos en un arbol de funcionales.<BR>
 *
 * <PRE>
 *
 *      Padre---> O
 *               /|\
 *              / | \
 *             /  |  \
 *  Hijos---> O   O   O
 *           (1) (2) (3)
 *
 * </PRE>
 *
 * La union entre dos nodos se representa como una referencia al
 * nodo padre y un entero que indica, en el padre, al numero de
 * de sub rama que ocupa el hijo que se identifica.<BR>
 *
 * Existe una excepcion a esta regla cuando el nodo identificado
 * es la raiz de un arbol (no tiene padre); en este caso, la
 * referencia (normalmente del padre) hace referencia al nodo
 * identificado y el numero de hijo es igual a cero.<BR>
 *
 * @author Antonio Pena
 * @version may-1997
 */
   public class AtomLink extends Object {

   /**
    * Apunta al padre del nodo buscado.<BR> */
      protected Funcional    padre = null;

   /**
    * Identifica al numero de sub rama en el padre que
    * ocupa el nodo buscado.<BR> */
      protected int          hijo  = 0;

   /**
    * La profundidad a la que se encuentra el nodo identificado.<BR> */
      protected int          prof = 0;



   /**
    * Construye una liga a partir de un nodo, su numero de hijo
    * y una profundidad.<BR> */
      public AtomLink( Funcional pad, int hij, int prf ) {
         padre = pad;
         hijo = hij;
         prof = prf;
      }


   /**
    * @return El padre del nodo identificado.<BR> */
      public Funcional getPadre() { return padre; }


   /**
    * @return El nodo identificado.<BR> */
      public Funcional getHijo() {
         if ( padre != null  &&  hijo > 0 )
            return padre.getOp( hijo );
         else
            return null;
      }


   /**
    * @return El numero de sub rama que el hijo ocupa en el padre.<BR> */
      public int getIHijo() { return hijo; }


   /**
    * @return La profundidad del nodo buscado.<BR> */
      public int getProf() { return prof; }


   /**
    * Sustituye al padre de la liga.<BR> */
      public void setPadre( Funcional pad ) { padre = pad; }


   /**
    * Sustituye al hijo de la liga.<BR> */
      public void setIHijo( int hij ) { hijo = hij; }
 

   /**
    * Sustituye la profundidad del nodo.<BR> */
      public void setProf( int prf ) { prof = prf; }
   }

