
// Parte del paquete de GP
   package GP;

// Uso de vectores por aqui
   import java.util.Vector;

/**
 * La clase <B>Especie</B> contiene los parametros de
 * construccion para los individuos de una poblacion.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class Especie extends Object {

   /**
    * Un nombre para la Especie.<BR> */
      protected String      name = null;

   /**
    * Las caracteristicas de construccion para la rama principal
    * de programa (rpb) del individuo.<BR> */
      protected TipoRama    def_rpb = null;

   /**
    * Un vector para contener las caracteristicas de
    * construccion de cada una de las ADFs del individuo.<BR> */
      protected Vector      defs_adfs = null;



   /**
    * Constructor totalmente parametrizado.<BR>
    *
    * @param nomb
    *    El nombre para el especificador de la especie.<BR>
    * @param b
    *    Determina si la rama de programa de los individuos debe ser
    *    balanceada.<BR>
    * @param pi
    *    Especifica la profundidad inicial maxima de la rama de programa.<BR>
    * @param pm
    *    Especifica la profundidad maxima absoluta de la rama de programa.<BR>
    * @param t
    *    Especifica los nombres de las clases que implementan funcionales
    *    terminales para la rama principal.<BR>
    * @param nt
    *    Especifica los nombres de las clases que implementan funcionales
    *    NO terminales para la rama principal.<BR>
    *
    * @exception ClassNotFoundException
    *    Se genera si alguna de las clases terminales o no terminales
    *    del usuario no es encontrada.<BR> */
      public Especie(  String nomb,
                      boolean b,
                          int pi,
                          int pm,
                       String t[],
                       String nt[] ) 
       throws ClassNotFoundException 
      {
         this.name = nomb;

      // Fabrica la definicion de la Rama basado en los parametros
         this.def_rpb = new TipoRama( b,
                                      pi, pm,
                                      new FuncionalFactory( t ),
                                      new FuncionalFactory( nt ));

      // inicializa el vector de ADFs
         this.defs_adfs = new Vector();
      }

   /**
    * Constructor que permite crear una especie a partir
    * de una definicion existente para la rama principal.<BR>
    *
    * @param nomb
    *    El nombre para el especificador de la especie.<BR>
    * @param def_rama
    *    Especificacion para la rama principal de los individuos.<BR> */
      public Especie( String nomb, TipoRama def_rama ) {
         
         this.name = nomb;
         this.def_rpb = def_rama;
         this.defs_adfs = new Vector();
      }



   /**
    * Regresa una referencia a la especificacion de la rama principal
    * de los individuos de la especie.<BR>
    *
    * @return Una referencia de tipo <B>TipoRama</B> que contiene
    *         las caracteristicas de las ramas principales de
    *         los individuos de la especie.<BR> */
      public TipoRama getTipoRPB() { return this.def_rpb; }


   /**
    * El numero de ADFs implementadas en la especie.<BR> */
      public int getNADFs() {
         return this.defs_adfs.size();
      }


   /**
    * Devuelve la definicion del n-esimo ADF de la especie.<BR>
    *
    * @param n
    *    El numero de definicion de ADF a obtener (<B>n</B> > 1).<BR>
    *
    * @return Una referencia a la definicion del n-esimo ADF.<BR> */
      public TipoADF getTipoADF( int n ) {
         return (TipoADF) this.defs_adfs.elementAt( n-1 );
      }


   /**
    * Permite agregar una definicion de ADF a la especie.<BR>
    *
    * @param b
    *    Determina si la rama del ADF debe ser balanceada.<BR>
    * @param pi
    *    Especifica la profundidad inicial maxima de la rama de programa.<BR>
    * @param pm
    *    Especifica la profundidad maxima absoluta de la rama de programa.<BR>
    * @param nt
    *    Especifica los nombres de las clases que implementan funcionales
    *    NO terminales para utilizar en el ADF.<BR>
    * @param nomb
    *    Es el nombre de la clase que implementa el ADF.<BR>
    *
    * @exception ClassNotFoundException
    *    Se genera si alguna de las clases no terminales
    *    para la ADF no es encontrada.<BR> */
      public void addADF( boolean b, 
                              int pi, 
                              int pm, 
                           String nt[],
                              int nargs,
                           String nomb ) 
         throws ClassNotFoundException
      {

      // Crea la definicion del ADF
         TipoADF def_adf = new TipoADF( b, pi, pm,
                                        new FuncionalFactory( nt ),
                                        nargs, nomb );

      // Agregalo al vector de ADFs de la especie
         this.defs_adfs.addElement( def_adf );
      }


   /**
    * Permite agregar una definicion de ADF a la especie,
    * basandose en una especificacion de ADF existente.<BR>
    *
    * @param def
    *    Es una especificacion de ADF (<B>TipoADF</B>).<BR> */
      public void addADF( TipoADF def ) { this.defs_adfs.addElement( def ); }


   /**
    * Produce una representacion en <B>String</B> de los
    * datos de la especie.<BR> */
      public String toString() {
      
         String str = new String("Especie." + this.name + " (" );

         str = str.concat("\nRPB");
         str = str.concat("\n" + this.def_rpb );
         str = str.concat("\n");
        
         str = str.concat("\nADFS");
         for (int i=0; i<this.defs_adfs.size(); i++) {
            str = 
             str.concat("\n" + (TipoADF) this.defs_adfs.elementAt(i) );
            str = str.concat("\n");
         } 
         str = str.concat("\n)");

         return str.toString();
      }
   }

