
// Parte del paquete de GP
   package GP;

/**
 * La clase <B>Evaluador</B> es responsable por la invocacion
 * de entrenadores sobre grupos de individuos para obtener su
 * calificacion de adecuacion.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class Evaluador {
   
   /**
    * Esta es la clase del entrenador a utilizar en el problema.<BR> */
      protected Class       cls_trainer;
      

   /**
    * El constructor crea un evaluador usando un
    * <B>Entrenador</B> de la clase <B>nomb_c</B>.<BR>
    *
    * @param nomb_c
    *    El nombre de la clase a utilizar como entrenador.<BR>
    *
    * @exception ClassNotFoundException
    *    Se genera si la clase del entrenador no existe.<BR> */
      public Evaluador( String nomb_c )
       throws ClassNotFoundException
      {
      // Crea una clase que hable de la clase del entrenador
         cls_trainer = Class.forName( nomb_c );
      }


   /**
    * Efectua la evaluacion de la poblacion contra el entrenador
    * del problema.<BR>
    *
    * @param pob
    *    Es la poblacion de individuos a evaluar.<BR>
    * @param g_size
    *    Indica el numero de individuos a ser evaluados
    *    en forma simultanea por el entrenador.<BR>
    * @param max_trainers
    *    Indica el numero de entrenadores a ejecutar en
    *    forma simultanea a traves de <B>Threads</B>.<BR>
    *
    * @exception InstantiationException
    *    Se genera si la clase entrenadora del usuario
    *    no puede ser instanciada.<BR>
    * @exception IllegalAccessException
    *    Se genera bajo un error de proteccion al acceder
    *    a la clase entrenadora del usuario.<BR> */
      public void evalua( Poblacion pob, int g_size, int max_trainers )
       throws IllegalAccessException, InstantiationException
      {
      
      // El numero de individuos en la poblacion
         int          n = pob.getNPoblacional();

      // La prioridad de los entrenadores por debajo de la nuestra
         int trainer_pri = Thread.currentThread().getPriority() - 1;

      // Un ThreadGroup para la evaluacion
         ThreadGroup trainer_group = new ThreadGroup( "Evaluacion" );
         trainer_group.setMaxPriority( trainer_pri );

      // Decide cuantos threads van a correr
         int max_threads = (int) n/g_size;
         if ( max_threads > max_trainers )   max_threads = max_trainers;



      // Hazlo para el numero de grupos que quepan en la poblacion
         Individuo[]  grupo = new Individuo[ g_size ];
         for (int i=1; i<=(int) n/g_size; i++) {

         // Espera mientras este corriendo el maximo numero de entrenadores
            while ( trainer_group.activeCount() >= max_threads ) {
               synchronized ( trainer_group ) {
                  try   { trainer_group.wait( 1000 ); }
                  catch (InterruptedException e) { }
               }
            }

         // Crea un grupo
            for (int j=0; j<g_size; j++)
               grupo[ j ] = pob.getIndividuo( 1 + (i-1)*g_size + j );

         // Crea un nuevo entrenador y dale sus conejillos de indias
            Entrenador trainer = (Entrenador) cls_trainer.newInstance();
            if ( i == 1 )   trainer.startPobEval( pob );
            trainer.setCobayos( grupo );

         // Crea un thread con este entrenador y echalo a volar
            Thread trainer_thread = new Thread( trainer_group, trainer );
            trainer_thread.setPriority( trainer_pri );
            trainer_thread.start();
         }

      // Aguantate hasta que termine todo el mundo
         Thread[] trainers = new Thread[ trainer_group.activeCount() ];
         trainer_group.enumerate( trainers, true );
         for (int i=0; i<trainers.length; i++)
            if ( trainers[ i ] != null )
               try { trainers[ i ].join(); }
               catch (InterruptedException e) { }

      // Chutate al grupo de threads
         trainer_group.destroy();
         

      // Obten la suma de adecuaciones ajustadas de la poblacion
         float sum_fa = 0;
         for (int i=1; i<=n; i++)
            sum_fa += pob.getIndividuo( i ).getFa();

      // Calcula la Adecuacion Normalizada de cada individuo
         for (int i=1; i<=n; i++) {
            Individuo ind = pob.getIndividuo( i );
            ind.setFn( ind.getFa() / sum_fa );
         }
      }
   }

