
// Parte del paquete de GP
   package GP;

/**
 * Esta es la clase utilizada por la maquina de GP para manipular los
 * nodos de un arbol de funcionales.<BR>
 *
 * Todos los funcionales usados para la solucion de un problema deben
 * descender directa o indirectamente de esta clase.<BR>
 *
 * @author Antonio Pena
 * @version may-1997
 */
   public abstract class Funcional extends Object implements Cloneable {

   /**
    * El arreglo para los operandos del funcional.<BR>
    *
    * El constructor de las clases que desciendan de <B>Funcional</B>
    * deben instanciar este arreglo con una longitud determinada.<BR>
    * */
      protected Funcional[]    operandos = null;



   /**
    * Produce una copia "profunda" del funcional.<BR> */
      public abstract Object clone(); 


   /**
    * Indica el numero de operandos requeridos por el funcional.<BR>
    * */
      public int getNOp() {
         if ( operandos != null )   return operandos.length;
         else                       return 0;
      }


   /**
    * Devuelve alguno de los operandos del funcional.<BR> */
      public Funcional getOp( int n ) { return operandos[ n-1 ]; }


   /**
    * Sustituye alguno de los operandos del funcional.<BR>
    *
    * @param n
    *    El numero de operando a sustituir (<B>nO</B> > 0).<BR>
    * @param op
    *    El operando por el cual debe ser sustituido.<BR> */
      public void setOp( int n, Funcional op ) { operandos[ n-1 ] = op; }



   /**
    * Cuenta el numero de nodos en un arbol de funcionales.<BR> */
      public int getNNodos() {

      // Este nodo cuenta (como uno)
         int         n = 1;
 
      // Sumale los nodos de todas mis sub ramas
         for (int i=1; i<=getNOp(); i++)
            n += getOp(i).getNNodos();

         return n;
      }


   /**
    * Devuelve la profundidad maxima de una arbol de funcionales.<BR>
    * */
      public int getProf() { return getProf( 1, new Entero( 0 ) ); }
      protected int getProf( int p_act, Entero p_max ) {

      // Esta puede ser la profundidad maxima
         if ( p_act > p_max.valor )   p_max.valor = p_act;

      // Explora por cada una de mis subramas
         for (int i=1; i<=getNOp(); i++)
            getOp( i ).getProf( p_act+1, p_max );

         return p_max.valor;
      }


   /**
    * Crea una referencia a un nodo en un arbol y a su padre.<BR>
    *
    * @param n
    *    El numero de nodo que se desea encontrar.<BR>
    *
    * @return Una instancia de la clase <B>AtomLink</B> que identifica
    *         al nodo buscado.<BR> */
      public AtomLink buscaNodo( int n ) {
         return buscaNodo( new Entero( n ), 1 );
      }
      protected AtomLink buscaNodo( Entero n, int prof ) {
         AtomLink   el_bueno = null;

      // Nos pasamos de tueste?
         if ( n.valor < 1 )   return null;

      // El valor solo puede ser uno para la raiz de un arbol
         if ( n.valor == 1 )   return new AtomLink( this, 0, 1 );

      // Cuenta a este nodo en la busqueda
         --n.valor;

      // Busca en cada uno de mis hijos directos
         for (int i=1; i<=getNOp(); i++) {

         // El nodo buscado es mi i-esimo hijo
            if (n.valor == 1)
               return new AtomLink( this, i, prof+1 );

         // Busca en el resto de mis sub ramas
            if ((el_bueno = getOp(i).buscaNodo(n, prof+1)) != null)
               return el_bueno;
         }

      // Regresa lo encontrado
         return el_bueno;
      }


   /**
    * Por default, un funcional base no es argumento de una ADF.<BR>
    *
    * La clase <B>ArgADF</B> sobreescribe este metodo para indicar que
    * es argumento de una ADF.<BR>
    *
    * @see GP.ArgADF */
      public int getNArg() { return 0; }


   /**
    * Este metodo sustituye en un arbol de funcionales a aquellos
    * nodos con un valor de <B>getNArg()</B> mayor a cero por su
    * operando correspondiente en el arreglo <B>op</B>.<BR>
    *
    * @param op
    *    El conjunto de operandos a sustituir.<BR> */
      public Funcional expandeADF( Funcional[] op ) {
 
      // Soy un placeholder de ADF, regresa la subrama que me corresponde
         if ( getNArg() > 0 )   return op[ getNArg()-1 ];
 
      // Si no soy un placeholder, buscale en mis sub ramas
         for (int i=1; i<=getNOp(); i++)
            setOp( i, getOp( i ).expandeADF( op ) );
 
         return this;
      }
   }

