
// Parte del paquete de GP
   package GP;

/**
 * Esta clase implementa el motor de la maquina de GP.<BR>
 *
 * Se encarga de evaluar, ordenar, cruzar, mutar y reproducir
 * a las poblaciones en evolucion.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class Motor {
      
      private Poblacion   p;
      private Evaluador   e;
      private Especie     esp;
      private int         max_cnt;


      public Motor(      int rnd_seed,
                         int nInd,
                     Especie especie,
                      String metodo )
       throws ClassNotFoundException
      {

      // Inicia el generador de numeros aleatorios
         Random.init( rnd_seed );

      // Guarda la especie
         esp = especie;

      // Crea la poblacion
         this.p = new Poblacion( nInd, especie, metodo );
      }


      public void run( int nGen, int g_size,
                       float pcruza, float pmutacion, float preprod,
                       String entrenador, int max_entrenador )
       throws ClassNotFoundException,
              InstantiationException,
              IllegalAccessException
      {
         int generacion = 0;
         Poblacion nueva;
         do {
            generacion++;
            e = new Evaluador( entrenador );
            System.out.print( "E" );
            e.evalua( this.p, g_size, max_entrenador );
            System.out.print( "O " );
            this.p.sort();
            this.display(this.p, generacion);
            nueva = new Poblacion( p.getNPoblacional() );
            System.out.print( "C" );
            this.p.cruza( pcruza, nueva );
            System.out.print( "M" );
            this.p.muta( pmutacion, nueva );
            System.out.print( "R" );
            this.p.reproduce( preprod, nueva );
            this.p = nueva;
         } while (generacion < nGen);
      }


      private void display( Poblacion p, int generacion ) {
         int n = p.getNPoblacional();
         int best_ind = 0;
         float max_fn, max_fa, sum_fa, avg_fa;
         max_cnt = 0;
         int prof_max = 0;
         int nodos_max = 0;
         float fa, fn;
         max_fn = max_fa = sum_fa = 0;
         for (int i=1; i<=n; i++) {
            fa = p.getIndividuo(i).getFa();
            fn = p.getIndividuo(i).getFn();
            sum_fa += fa;
            if (fn > max_fn) {
                best_ind = i;
                prof_max = this.p.getIndividuo(i).getRPB().getProf();
                nodos_max = this.p.getIndividuo(i).getNNodos();
                max_fn = fn;
            }
            if (fa > max_fa) {
                max_fa = fa; max_cnt = 0;
            }
            if (fa == max_fa) { ++max_cnt; }
         }
         avg_fa = sum_fa / n;
         System.out.print("Gen: " + generacion);
         System.out.print("x" + this.p.getNPoblacional() );
         System.out.print(" FaMax (" + nodos_max + ", " + prof_max + "): ");
         System.out.print(max_fa + "x" + max_cnt);
         System.out.println(" FaAvg: " + avg_fa);
         // System.out.println(" FaSum: " + sum_fa);

      // El mejor de la generacion
         System.err.print(generacion + ": ");
         System.err.println(this.p.getIndividuo(best_ind).toString());
      }
   }

