
// Parte del paquete de GP
   package GP;

// Uso de vectores por aqui
   import java.util.Vector;

/**
 * La clase <B>Poblacion</B> encapsula a un <B>Vector</B> de
 * individuos para su manipulacion en la maquina de GP.<BR>
 *
 * @author Antonio Pena
 * @version may-1997
 */
   public class Poblacion extends Object {

   /**
    * El vector para contener individuos.<BR> */
      private Vector         individuos;

    
   /**
    * Constructor para poblaciones iniciales.<BR>
    *
    * @param num_i
    *    El numero de individuos a manejar en la poblacion.<BR>
    * @param esp
    *    Informacion de construccion de individuos.<BR>
    * @param metodo
    *    Metodo de poblacion.<BR>
    *    <B>metodo</B>=="FULL"  Todos los individuos son construidos
    *                           con su <B>rpb</B> balanceada hasta
    *                           la profundidad inicial.<BR>
    *    <B>metodo</B>=="GROW"  Las <B>rpb</B> de los individuos
    *                           son desbalanceadas.<BR>
    *    <B>metodo</B>=="FRAMP" Las <B>rpb</B> de los individuos
    *                           son balanceadas y de profundidades
    *                           crecientes.<BR>
    *    <B>metodo</B>=="GRAMP" Las <B>rpb</B> de los individuos
    *                           son desbalanceadas y de profundidades
    *                           crecientes.<BR>
    *    <B>metodo</B>=="HRAMP" Las <B>rpb</B> de los individuos
    *                           son balanceadas y desbalanceadas
    *                           y de profundidades crecientes.<BR> */
      public Poblacion( int num_i, Especie esp, String metodo ) {
   
      // Inicia el vector de individuos
         individuos = new Vector( num_i );

      // Profundidad para iniciar la generacion
         double      s_prof = 0;

      // Delta de profundidad por individuo
         double      pdelta = 0;

      // Umbral probabilistico de balanceo
         double      halved = 0;


      // De acuerdo al metodo de construccion
                if ( metodo.compareTo( "FULL"  ) == 0 ) {
                   s_prof = esp.getTipoRPB().getProfIni();
                   pdelta = 0;
                   halved = 1.0;
         } else if ( metodo.compareTo( "GROW"  ) == 0 ) {
                   s_prof = esp.getTipoRPB().getProfIni();
                   pdelta = 0.0;
                   halved = 0.0;
         } else if ( metodo.compareTo( "FRAMP" ) == 0 ) {
                   s_prof = 2;
                   pdelta = (double)
                            (esp.getTipoRPB().getProfIni() - 1) / num_i;
                   halved = 1.0;
         } else if ( metodo.compareTo( "GRAMP" ) == 0 ) {
                   s_prof = 2;
                   pdelta = (double)
                            (esp.getTipoRPB().getProfIni() - 1) / num_i;
                   halved = 0.0;
         } else if ( metodo.compareTo( "HRAMP" ) == 0 ) {
                   s_prof = 2;
                   pdelta = (double)
                            (esp.getTipoRPB().getProfIni() - 1) / num_i;
                   halved = 0.5;
         }

      // Construye la poblacion
         for (int i=0; i<num_i; i++) {
            Individuo nuevo = new Individuo( esp, (int) (s_prof + pdelta*i),
                                             Random.randomFloat() < halved );
            this.individuos.addElement( nuevo );
         }
      }


   /**
    * Constructor de poblaciones vacias, usado para la
    * creacion de poblaciones de generaciones posteriores.<BR>
    *
    * @param num_i
    *     Numero de individuos a manejar en la poblacion.<BR> */
      public Poblacion( int num_i ) {
         individuos = new Vector( num_i );
      }


   /**
    * El numero de individuos en la poblacion.<BR> */
      public int getNPoblacional() {
         return this.individuos.size();
      }


   /**
    * Ordena el conjunto de individuos de la poblacion
    * utilizando el algoritmo de QuickSort de la clase <B>Sort</B> */
      public void sort() {
         Sort.vQSort( this.individuos );
      }


   /**
    * Agrega un individuo a la poblacion.<BR> */
      public void addIndividuo( Individuo ind ) {
         this.individuos.addElement( ind );
      }


   /**
    * Devuelve una referencia al n-esimo individuo de la poblacion.<BR>
    *
    * @param n
    *    El parametro debe estar en el rango de individuos
    *    de la poblacion.<BR>
    *
    * @return <B>null</B> si <B>n</B> esta fuera de rango.<BR> */
      public Individuo getIndividuo( int n ) {
         return (Individuo) this.individuos.elementAt( n-1 );
      }


   /**
    * Sustituye al n-esimo individuo de la poblacion.<BR> */
      public void setIndividuo( Individuo ind, int n ) {
         this.individuos.setElementAt( ind, n-1 );
      }


   /**
    * Elimina el individuo n.<BR>
    *
    * @param n
    *    El parametro debe estar en el rango de individuos
    *    de la poblacion.<BR> */
      public void delIndividuo( int n ) {
         this.individuos.removeElementAt( n-1 );
      }
    

   // Busca al individuo para el cual la sumatoria de
   // Normalized fitness es mayor que el numero otorgado
      public int fitnessSeek( float fn ) {
        int     i, np;
        float sfn;


        i = 0;  np = getNPoblacional();  sfn = 1;
        while (( i < np ) && ( sfn > fn )) {
           sfn -= getIndividuo(i+1).getFn();
           if (sfn > fn)  ++i;
        }

        return i < np ? i+1 : np;
      }

    
   // Busca al individuo para el cual la sumatoria de
   // Normalized fitness es mayor que el numero otorgado
      public int replaceSeek( float fn ) {
        int     i, np;
        float sfn;


        i = 0;  np = getNPoblacional();  sfn = 0;
        while (( i < np ) && ( sfn < fn )) {
           sfn += (1 - getIndividuo(i+1).getFn());
           if (sfn < fn)  ++i;
        }

        return i < np ? i+1 : 1;
      }


   /**
    * Implementa el proceso de cruzamiento sexuado de individuos de
    * la poblacion.<BR>
    *
    * @param p_cruza
    *    Es el porcentaje de la poblacion que se somete al
    *    proceso de cruzamiento. Su rango es <B>[0, 1]</B>.<BR>
    * @param pob
    *    Es la poblacion en la que debe insertarse el individuo mutado.<BR> */
      public void cruza( float p_cruza, Poblacion pob ) {

      // El numero de elementos en la poblacion
         int n = this.getNPoblacional();

      // Echate las cruzas para el numero de individuos requeridos
         for (int i=0; i<(int) n*p_cruza/2; i++) {

         // Elige dos individuos para aparearse
            int num_a = fitnessSeek( Random.randomFloat() );
            int num_b = fitnessSeek( Random.randomFloat() );

         // Quienes son?
            Individuo ind_a = getIndividuo( num_a );
            Individuo ind_b = getIndividuo( num_b );

         // Follad !
            ind_a.cruzamiento( ind_b, pob );
         }
      }


   /**
    * Implementa el proceso de mutacion de individuos de la poblacion.<BR>
    *
    * @param p_mutacion
    *    Es el porcentaje de la poblacion que se somete al
    *    proceso de mutacion. Su rango es <B>[0, 1]</B>.<BR>
    * @param pob
    *    Es la poblacion en la que debe insertarse el individuo mutado.<BR> */
      public void muta( float p_mutacion, Poblacion pob ) {

      // El numero de elementos en la poblacion
         int n = this.getNPoblacional();

      // Muta al numero de individuos requerido
         for (int i=0; i<(int) n*p_mutacion; i++) {

         // Elige el individuo a mutar
            int num = fitnessSeek( Random.randomFloat() );

         // Quien es?
            Individuo ind = getIndividuo( num );

         // Muta a este chalan !
            ind.mutacion( pob );
         }
      }


   /**
    * Implementa el proceso de reproduccion de individuos de la pobalcion.<BR>
    *
    * @param p_reprod
    *    Es el porcentaje de la poblacion que se somete al
    *    proceso de reproduccion. Su rango es <B>[0, 1]</B>.<BR>
    * @param pob
    *    Es la poblacion en la que debe insertarse el resultado de
    *    la reproduccion.<BR> */
      public void reproduce( float p_reprod, Poblacion pob ) {

      // El numero de elementos en la poblacion
         int n = this.getNPoblacional();

      // Reproduce al porcentaje de la poblacion
         for (int i=0; i<(int) n*p_reprod; i++) {

         // Elige el individuo a reproducir
            int num = fitnessSeek( Random.randomFloat() );

         // Quien es?
            Individuo ind = getIndividuo( num );

         // Ponlo en la nueva poblacion
            pob.addIndividuo( ind );
         }
      }
   }

