
// Parte del paquete de GP
   package GP;

/**
 * La clase que implementa una rama de funcionales.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class Rama extends Object implements Cloneable {

   /**
    * El nodo raiz de la rama.<BR> */
      protected Funcional    raiz = null;
      
   /**
    * El numero de nodos que conforman la rama.<BR> */
      protected int          nodos_totales = 0;

   /**
    * La profundidad de la estructura.<BR> */
      protected int          prof = 0;
      
   /**
    * Los parametros utilizados en la construccion.<BR> */
      protected TipoRama     tipo = null;



   /**
    * Un constructor vacio.<BR> */
      public Rama() { }

   /**
    * Construye una rama de acuerdo a su definicion de tipo.<BR>
    *
    * @param tip
    *    Una instancia con las especificaciones de
    *    construccion para la rama.<BR> */
      public Rama( TipoRama tip ) {
         tipo = tip;

      // Crea el arbol en forma recursiva
         raiz = nuevaRama( 1, tipo.getProfIni(), tipo.getBalance() );
      }

   /**
    * Construye una rama de acuerdo a su definicion de tipo
    * pero con caracteristicas de profundidad y balanceo especiales.<BR>
    *
    * @param tip
    *    Una instancia con las especificaciones de
    *    construccion para la rama.<BR>
    * @param prf
    *    La profundidad maxima de construccion.<BR>
    * @param bal
    *    El indicador de balanceo en la rama.<BR> */
      public Rama( TipoRama tip, int prf, boolean bal ) {
         tipo = tip;

      // Crea el arbol en forma recursiva
         raiz = nuevaRama( 1, prf, bal );
      }


   /**
    * Genera una copia "profunda" de la rama.<BR> */
      public Object clone() {
         Rama clon = new Rama();

      // OJO: el tipo no se clona, solo se copia la referencia
         clon.tipo = this.tipo;

      // Clona los nodos de la rama
         clon.raiz = (Funcional) this.raiz.clone();

      // El numero de nodos en la rama y la profundidad
         clon.prof = this.prof;
         clon.nodos_totales = this.nodos_totales;

         return clon;
      }


   /**
    * Devuelve el numero de nodos en la rama.<BR> */
      public int getNNodos() { return nodos_totales; }


   /**
    * Devuelve la profundidad actual de la rama.<BR> */
      public int getProf() { return prof; }


   /**
    * Devuelve el arbol contenido en la rama.<BR>
    *
    * @return Una referencia al nodo raiz del arbol de composicion
    *         contenido en la rama.<BR> */
      public Funcional getRaiz() { return raiz; }


   /**
    * Permite sustituir el arbol de composicion de la rama.<BR>
    *
    * @param nueva
    *    Es una referencia a la nueva raiz de la rama.<BR>
    *
    * @return Una referencia a la raiz sustituida.<BR> */
      public Funcional setRaiz( Funcional nueva ) {
         return sustituye( new AtomLink( raiz, 0, 1 ), nueva );
      }


   /**
    * Permite sustituir una seccion de la rama.<BR>
    *
    * @param corte
    *    Es un identificador de punto de corte en la rama.<BR>
    * @param nuevo
    *    Es una referencia a la nueva sub rama a instalar.<BR>
    *
    * @return Una referencia a la subrama sustituida.<BR> */
      public Funcional sustituye( AtomLink corte, Funcional nuevo ) {

      // La actual subrama a sustituir
         Funcional actual = corte.getIHijo() > 0 ?
                            corte.getHijo() : corte.getPadre();

      // Tal vez quieren cambiar desde la raiz
         if ( corte.getIHijo() == 0 )
            raiz = nuevo;
         else
            corte.getPadre().setOp( corte.getIHijo(), nuevo );

      // Recuenta los nodos y la profundidad
         prof = raiz.getProf();
         nodos_totales += nuevo.getNNodos() - actual.getNNodos();

         return actual;
      }


   /**
    * Crea una nueva rama de funcionales.<BR>
    *
    * @param p_act
    *    Es el indicador de la profunfidad actual del proceso
    *    en el descenso recursivo.<BR>
    * @param p_max
    *    Es la profundidad maxima con la que se debe construir
    *    la rama.<BR>
    * @param bal
    *    Indica si la rama debe ser o NO balanceada.<BR> */
      private Funcional nuevaRama( int p_act, int p_max, boolean bal ) {
      
      // Elige el grupo del cual se instancia el atomo
      //   T -> Terminal,   N -> No terminal
      // OJO: En arboles no balanceados echamos un volado !
         char       grupo;
         if ( p_act >= p_max )                       grupo = 'T';
         else if ( p_act == 1 )                      grupo = 'N';
         else if ( bal )                             grupo = 'N';
         else grupo = Random.randomFloat() < 0.5f ?  'T'  :  'N';

      // Crea el atomo
         Funcional nodo = null;
         switch (grupo) {
            case 'T': nodo = tipo.getTerminales().nuevo(
                                Random.randomInt( tipo.getTerminales().n() ));
                      break;
            case 'N': nodo = tipo.getNoTerminales().nuevo(
                                Random.randomInt( tipo.getNoTerminales().n() ));
                      break;
         }

      // Ahora crea las sub ramas del atomo
         for (int i=1; i<=nodo.getNOp(); i++)
            nodo.setOp( i, nuevaRama( p_act+1, p_max, bal ) );

      // Suma uno al total de nodos en la rama
         nodos_totales++;

      // La profundidad maxima de la rama
         if ( p_act > prof )   prof = p_act;
         
      // Regresa el atomo creado
         return nodo;
      }


   /**
    * Identifica el enlace de un nodo con su padre en el
    * arbol de composicion de la rama.<BR>
    *
    * @param n
    *    El numero de nodo deseado.<BR>
    *
    * @return La referencia devuelta por el metodo <B>buscaNodo</B>
    *         de la clase <B>Funcional</B>.<BR> */
      public AtomLink buscaNodo( int n ) {
         return raiz.buscaNodo( n );
      }
   }

