
// Parte del paquete de GP
   package GP;

/**
 * Encapsula las caracteristicas de una
 * rama que implementa una ADF.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class TipoADF extends TipoRama implements Cloneable {

   /**
    * El nombre de la clase que SIMBOLIZA la ADF.<BR> */
      protected String                     nombre;

   /**
    * El numero de argumentos que utiliza la ADF.<BR> */
      protected int                        nargs;
   
   /**
    * Este es el conjunto de terminales para toda ADF.<BR> */
      protected static String[]            t_adf = { "GP.ArgADF" };
      protected static FuncionalFactory    t = null;



   /**
    * Un constructor vacio.<BR> */
      public TipoADF() { }

   /**
    * Un constructor totalmente parametrizado.<BR>
    *
    * @param b
    *    Indica si la ADF es balanceada.<BR>
    * @param pi
    *    La profundidad inicial maxima de la ADF.<BR>
    * @param pm
    *    La profundidad maxima absoluta de la ADF.<BR>
    * @param nt
    *    Una instancia capaz de generar los funcionales
    *    NO terminales para ser usados en la ADF.<BR>
    * @param na
    *    El numero de argumentos utilizados por la ADF.<BR>
    * @param nomb
    *    El nombre de la clase que implementa la ADF.<BR> */
      public TipoADF( boolean b, int pi, int pm,
                      FuncionalFactory nt,
                      int na, String nomb ) {

      // Llama al constructor de la superclase TipoRama
         super( b, pi, pm, null, nt );

      // Fija el unico funcional terminal posible para una ADF
         try   { if ( t == null )   t = new FuncionalFactory( t_adf ); }
         catch (ClassNotFoundException e) { }
         setTerminales( t );

      // Nombre y numero de argumentos
         nombre = nomb;
         nargs = na;
      }



   /**
    * Genera un clon de la especificacion de la ADF.<BR> */
      public Object clone() {
         return new TipoADF( getBalance(),
                             getProfIni(), getProfMax(),
                             getNoTerminales(),
                             nargs, new String( nombre ) );
      }


   /**
    * El nombre del tipo de ADF.<BR> */
      public String getNombre() { return nombre; }

   /**
    * El numero de argumentos de la ADF.<BR> */
      public int getNArgs() { return nargs; }


   /**
    * Fija el nombre de la clase que implementa la ADF.<BR> */
      public void setNombre( String nomb ) { nombre = nomb; }

   /**
    * Fija el numero de argumentos de la ADF.<BR> */
      public void setNArgs( int na ) { nargs = na; }


   /**
    * Genera una representacion <B>String</B> de la definicion
    * de la ADF.<BR> */
      public String toString() {
         return "ADF (" + nombre + ") {"
              + "\n\t# args     : " + nargs
              + "\n\tbalanceado : " + ( getBalance() ? "si" : "no" )
              + "\n\tprof ini   : " + getProfIni()
              + "\n\tprof max   : " + getProfMax()
              + "\n\t" + getTerminales()
              + "\n\t" + getNoTerminales()
              + "\n}";
      }
   }

