
// Parte del paquete de GP
   package GP;

/**
 * Encapsula las caracteristicas de una rama de funcionales.<BR>
 *
 * Toda rama tiene una profundidad de creacion (<B>prof_ini</B>) y
 * una profundidad maxima permitida en operacion
 * (<B>prof_max</B>).<BR>
 *
 * Cuenta con un indicador (<B>balance</B>) que determina si
 * todas las sub ramas de la estructura deben alcanzar la
 * profundidad <B>prof_ini</B> al ser creadas.<BR>
 *
 * Los nodos de la rama son creados usando las instancias
 * generadas por las instancias de <B>FuncionalFactory</B>
 * referenciadas por * <B>terminales</B> y <B>noterminales</B>.<BR>
 *
 * @author Antonio Pena
 * @version jun-1997
 */
   public class TipoRama extends Object implements Cloneable {

   /**
    * Indicador de balanceo de la rama.<BR> */
      protected boolean             balance = false;
   
   /**
    * Profundidad de creacion para la rama.<BR> */
      protected int                 prof_ini = 0;

   /**
    * Profundidad maxima permitida.<BR> */
      protected int                 prof_max = 0;

   /**
    * Generador de nodos terminales.<BR> */
      protected FuncionalFactory    terminales = null;

   /**
    * Generador de nodos NO terminales.<BR> */
      protected FuncionalFactory    noterminales = null;



   /**
    * Un constructor vacio.<BR> */
      public TipoRama() { }

   /**
    * Un constructor totalmente parametrizado.<BR>
    *
    * @param b
    *    Indica si la rama es balanceada.<BR>
    * @param pi
    *    La profundidad inicial maxima de la rama.<BR>
    * @param pm
    *    La profundidad maxima permitida para la rama.<BR>
    * @param t
    *    Una instancia capaz de generar los funcionales
    *    terminales para ser usados en la rama.<BR>
    * @param nt
    *    Una instancia capaz de generar los funcionales
    *    NO terminales para ser usados en la rama.<BR> */
      public TipoRama( boolean b, int pi, int pm,
                       FuncionalFactory t, FuncionalFactory nt ) {
         balance = b;
         prof_ini = pi;
         prof_max = pm;
         terminales = t;
         noterminales = nt;
      }


    
   /**
    * Genera un clon de la especificacion de rama.<BR> */
      public Object clone() {
         return new TipoRama( balance,
                              prof_ini, prof_max,
                              terminales, noterminales);
      }


   /**
    * El indicador de contruccion balanceada para la rama.<BR> */
      public boolean getBalance() { return balance; }

   /**
    * La profundidad inicial para la rama.<BR> */
      public int getProfIni() { return prof_ini; }

   /**
    * La profundidad maxima para la rama.<BR> */
      public int getProfMax() { return prof_max; }

   /**
    * El <B>FuncionalFactory</B> para instanciar terminales.<BR> */
      public FuncionalFactory getTerminales() { return terminales; }

   /**
    * El <B>FuncionalFactory</B> para instanciar NO terminales.<BR> */
      public FuncionalFactory getNoTerminales() { return noterminales; }


   /**
    * Fija el indicador de contruccion balanceada para la rama.<BR> */
      public void setBalance( boolean b ) { balance = b; }

   /**
    * Fija la profundidad inicial para la rama.<BR> */
      public void setProfIni( int pi ) { prof_ini = pi; }

   /**
    * Fija la profundidad maxima para la rama.<BR> */
      public void setProfMax( int pm ) { prof_max = pm; }

   /**
    * Fija el <B>FuncionalFactory</B> para
    * instanciar terminales.<BR> */
      public void setTerminales( FuncionalFactory t ) {
         terminales = t;
      }

   /**
    * Fija el <B>FuncionalFactory</B> para
    * instanciar NO terminales.<BR> */
      public void setNoTerminales( FuncionalFactory nt ) {
         noterminales = nt;
      }


   /**
    * Genera una representacion <B>String</B> de
    * las definiciones de la rama.<BR> */
      public String toString() {
         return "Rama {"
              + "\n\tbalanceado : " + ( balance ? "si" : "no" )
              + "\n\tprof ini   : " + prof_ini
              + "\n\tprof max   : " + prof_max
              + "\n\t" + terminales
              + "\n\t" + noterminales
              + "\n}";
      }
   }

