
// Uso de stacks por aqui
   import java.util.Stack;

/**
 * Implementa la representacion de un poste de tablero
 * en el problema de torres de Hanoi.<BR>
 *
 * @author Antonio Pena
 * @version jul-1997
 */
   public class hnPoste extends Object {

   // Una pila de piezas
      Stack    piezas;


   /**
    * Un constructor vacio.<BR> */
      public hnPoste() { piezas = new Stack(); }

   /**
    * Produce una copia del poste.<BR> */
      public Object clone() {
         hnPoste clon = new hnPoste();

      // Copia todas las piezas del poste
         for (int i=0; i<this.piezas.size(); i++)
           clon.piezas.push( this.piezas.elementAt(i) );

         return clon;
      }


   /**
    * Devuelve el numero de piezas en un poste.<BR> */
      public int nPiezas() { return piezas.size(); }

   /**
    * Pon una pieza en el poste.<BR>
    *
    * @param pieza
    *    Es una referencia a la pieza a insertar.<BR> */
      public void ponPieza( hnPieza pieza ) { piezas.push( pieza ); }

   /**
    * Determina si el poste esta vacio.<BR> */
      public boolean vacio() { return piezas.empty(); }

   /**
    * Devuelve la pieza superior del poste sin removerla,
    * o <B>null</B> si el poste esta vacio.<BR> */
      public hnPieza piezaSup() {

         if ( ! piezas.empty() )
            return (hnPieza) piezas.peek();
         else
            return null;
      }

   /**
    * Devuelve y remueve la pieza superior del poste,
    * o <B>null</B> si el poste esta vacio.<BR> */
      public hnPieza quitaPieza() {

         if ( piezas.empty() )   return null;
         else                    return (hnPieza) piezas.pop();
      }

   /**
    * Determina si dos postes contienen las mismas piezas.<BR>
    *
    * @param cmp
    *    Es el poste contra el que se compara.<BR> */
      public boolean equals( hnPoste cmp ) {

      // El numero de piezas debe ser el mismo
         if ( this.piezas.size() != cmp.piezas.size() )   return false;

      // Compara todas las piezas de los postes
         for (int i=0; i<this.piezas.size(); i++) {
            hnPieza pieza_t = (hnPieza) this.piezas.elementAt( i );
            hnPieza pieza_c = (hnPieza)  cmp.piezas.elementAt( i );
            if ( ! pieza_t.equals( pieza_c ) )   return false;
         }

         return true;
      }

   /**
    * Regresa una representacion <B>String</B> del poste.<BR> */
      public String toString() {
         String str = new String( "Poste ->" );

      // Concatena todas las piezas del poste
         for (int i=0; i<piezas.size(); i++)
            str = str.concat( " " +
                              ((hnPieza) piezas.elementAt( i )).toString() );

         return str;
      }
   }

