
/**
 * Implementa la representacion de un tablero de torres de Hanoi.<BR>
 *
 * El tablero puede ser construido con diferente numero de postes
 * y lleva cuenta del numero de movimientos efectuados sobre sus
 * piezas.<BR>
 *
 * Tambien almacena una referencia al ultimo poste modificado.<BR>
 *
 * @author Antonio Pena
 * @version jul-1997
 */
   public class hnTablero extends Object {

   // Arreglo de postes en el tablero
      hnPoste[]    postes = null;

   // Numero de movimientos efectuados
      int          mov = 0;

   // El ultimo poste afectado en una movida
      hnPoste      ultimo = null;


   /**
    * Un constructor basico.<BR>
    *
    * @param n_postes
    *    Especifica el numero de postes en el tablero.<BR> */
      public hnTablero( int n_postes ) { postes = new hnPoste[ n_postes ]; }

   /**
    * Produce una copia del tablero.<BR> */
      public Object clone() {

      // El nuevo tablero
         hnTablero clon = new hnTablero( postes.length );

      // Duplica cada uno de los postes
         for (int i=0; i<postes.length; i++)
            if ( this.postes[ i ] == null )
               clon.postes[ i ] = null;
            else
               clon.postes[ i ] = (hnPoste) this.postes[ i ].clone();

      // Copia el numero de movimientos y el ultimo poste afectado
         clon.mov = this.mov;
         clon.ultimo = this.ultimo == null
                          ? null
                          : clon.postes[ this.idxPoste( this.ultimo ) ];

         return clon;
      }


   /**
    * Fija uno de los postes del tablero.<BR>
    *
    * @param n
    *    Es el numero de poste a sustituir.<BR>
    * @param poste
    *    Es el poste a poner en el tablero.<BR> */
      public void setPoste( int n, hnPoste poste ) { postes[ n ] = poste; }

   /**
    * Fija la referencia del ultimo poste modificado.<BR> */
      public void setUtlimo( hnPoste ult ) { ultimo = ult; }

   /**
    * Mueve una pieza del poste origen al poste destino.<BR>
    *
    * @param orig
    *    Identifica al poste del que debe tomarse la pieza.<BR>
    * @param dest
    *    Identifica al poste en el cual debe ponerse la pieza.<BR>
    * @return Una referencia al poste al que se movio la pieza.<BR> */
      public hnPoste muevePieza( hnPoste orig, hnPoste dest ) {

      // Los postes deben existir en este tablero
         if ( orig == null  ||  dest == null )   return null;
         if ( idxPoste( orig ) < 0  ||  idxPoste( dest ) < 0 )   return null;

      // El poste origen no puede estar vacio
         if ( orig.vacio() )   return null;

      // La pieza debe ser menor a la del poste destino
         if ( dest.piezaSup() != null
           && dest.piezaSup().compareTo( orig.piezaSup() ) <= 0 )
            return null;

      // Todo se ve bien ... mueve la pieza
         dest.ponPieza( orig.quitaPieza() );
         ++mov;   ultimo = dest;

      // El poste destino
         return orig;
      }

   /**
    * Devuelve el numero de piezas en el tablero.<BR> */
      public int nPiezas() {
         int cuenta = 0;

      // Suma las piezas en cada poste
         for (int i=0; i<postes.length; i++)
            cuenta += postes[ i ].nPiezas();

         return cuenta;
      }
   
   /**
    * Determina si dos tableros tienen la misma configuracion.<BR>
    *
    * @param cmp
    *    Es el tablero contra el que se compara.<BR> */
      public boolean equals( hnTablero cmp ) {

      // Deben tener el mismo numero de postes
         if ( this.postes.length != cmp.postes.length )   return false;

      // Compara cada uno de los postes
         for (int i=0; i<this.postes.length; i++) {

         // Los dos nulos ... son iguales !
            if ( this.postes[ i ] == null  &&  cmp.postes[ i ] == null )
               continue;

         // Uno solo nulo ... ya valio !
            if ( this.postes[ i ] == null  ||  cmp.postes[ i ] == null )
               return false;

         // Los dos llenos ... comparalos !
            if ( ! this.postes[ i ].equals( cmp.postes[i] ) )   return false;
         }

         return true;
      }

   /**
    * Regresa el numero de un poste en el tablero.<BR>
    *
    * @param poste
    *    Es una referencia al poste del tablero.<BR>
    *
    * @returns Un indice numerico para el poste identificado, o -1
    *          si el poste no esta en el tablero.<BR> */
      public int idxPoste( hnPoste poste ) {

      // Busca el poste en el tablero
         for (int i=0; i<postes.length; i++)
            if ( poste == postes[ i ] )   return i;

         return -1;
      }

   /**
    * Regresa una representacion <B>String</B> del tablero.<BR> */
      public String toString() {
         String str = new String( "Tablero:\n" );

      // Concatena todos los postes del tablero
         for (int i=0; i<postes.length; i++)
            str = str.concat( "   " + postes[ i ].toString() + "\n" );

         return str;
      }
   }

