
// Las clases del paquete de GP
   import GP.*;

/**
 * Esta clase es un evaluador disenado para evaluar a individuos
 * en la solucion de un numero de casos del problema de torres
 * de Hanoi.<BR>
 *
 * @author Antonio Pena
 * @version jul-1997
 */
   public class hnTrainer extends GP.Entrenador { 

   // El arreglo que contiene los ambientes seleccionados
      private static int             casos;
      private static hnAmbiente[]    ambientes;


   /**
    * Evalua a un individuo contra los casos almacenados
    * en el arreglo <B>ambientes</B>.<BR> */
      private double fd( Individuo ind ) {
         double calif = 0;


      // Evalua cada uno de los casos
         for (int i=0; i<ambientes.length; i++) {

         // Reproduce el ambiente para el trabajito
            hnAmbiente trabajo = (hnAmbiente) ambientes[ i ].clone();

         // El numero maximo de evaluaciones para el individuo
            int veces = 0;
            int max_eval = (int) Math.pow( 2, trabajo.origen.nPiezas() ) - 1;
            if ( ! trabajo.ya() )
               do {
                  ++veces;
                  ((hnNodo) ind.getRPB().getRaiz()).eval( ind, trabajo );
               } while ( ! trabajo.ya()
                     && veces <= max_eval );

         // Gana un punto si el ambiente esta solucionado
            if ( trabajo.ya() ) {
               calif += 0.75;
               if ( trabajo.mov_min == 0  ||  trabajo.origen.mov == 0 )
                  calif += 0.25;
               else
                  calif += 0.25
                         * (double) trabajo.mov_min / trabajo.origen.mov;
            }
         }

         return calif;
      }


   /**
    * La implementacion del metodo <B>fitness()</B> de la
    * clase abstracta <B>Entrenador</B>.<BR> */
      public float[] fitness() {

      // Para devolver la calificacion del individuo
         float[] calif = new float[ 0 ];

      
      // Evalua al chamaco
         double c_este = fd( cobayos[0] );
         cobayos[ 0 ].setFa( (float) ( 1.0f /
                                       ( 1 + hnTrainer.casos - c_este ) ) );

         return calif;
      }


   /**
    * Este metodo debe ser invocado antes de utilizar las
    * capacidades de evaluacion de la clase, ya que inicializa
    * los casos de solucion en el arreglo <B>ambientes</B>.<BR> */
      public static void init() {

      // El tablero origen
         hnTablero    org;

      // Un poste auxiliar
         hnPoste      p_aux;


      // Las piezas de los tableros
         hnPieza[] piezas = new hnPieza[ 6 ];
         piezas[ 0 ] = new hnPieza( 'A' );
         piezas[ 1 ] = new hnPieza( 'B' );
         piezas[ 2 ] = new hnPieza( 'C' );
         piezas[ 3 ] = new hnPieza( 'D' );
         piezas[ 4 ] = new hnPieza( 'E' );
         piezas[ 5 ] = new hnPieza( 'F' );

      // El numero de casos a evaluar
         casos = 6;
         ambientes = new hnAmbiente[ casos ];

      // Caso 1
         org = new hnTablero( 3 );
         p_aux = new hnPoste();
         org.setPoste( 0, p_aux );
         p_aux = new hnPoste();
         org.setPoste( 1, p_aux );
         p_aux.ponPieza( piezas[ 0 ] );
         p_aux = new hnPoste();
         p_aux.ponPieza( piezas[ 3 ] );
         p_aux.ponPieza( piezas[ 2 ] );
         p_aux.ponPieza( piezas[ 1 ] );
         org.setPoste( 2, p_aux );
         org.ultimo = org.postes[ 2 ];

         ambientes[ 0 ] = new hnAmbiente( org, org.postes[2], 1 );

      // Caso 2
         org = new hnTablero( 3 );
         p_aux = new hnPoste();
         org.setPoste( 0, p_aux );
         p_aux.ponPieza( piezas[ 1 ] );
         p_aux.ponPieza( piezas[ 0 ] );
         p_aux = new hnPoste();
         org.setPoste( 1, p_aux );
         p_aux = new hnPoste();
         p_aux.ponPieza( piezas[ 3 ] );
         p_aux.ponPieza( piezas[ 2 ] );
         org.setPoste( 2, p_aux );
         org.ultimo = org.postes[ 2 ];

         ambientes[ 1 ] = new hnAmbiente( org, org.postes[2], 3 );

      // Caso 3
         org = new hnTablero( 3 );
         p_aux = new hnPoste();
         org.setPoste( 0, p_aux );
         p_aux.ponPieza( piezas[ 1 ] );
         p_aux = new hnPoste();
         org.setPoste( 1, p_aux );
         p_aux.ponPieza( piezas[ 2 ] );
         p_aux = new hnPoste();
         org.setPoste( 2, p_aux );
         p_aux.ponPieza( piezas[ 3 ] );
         p_aux.ponPieza( piezas[ 0 ] );
         org.ultimo = org.postes[ 0 ];

         ambientes[ 2 ] = new hnAmbiente( org, org.postes[2], 5 );

      // Caso 4
         org = new hnTablero( 3 );
         p_aux = new hnPoste();
         org.setPoste( 0, p_aux );
         p_aux.ponPieza( piezas[ 3 ] );
         p_aux.ponPieza( piezas[ 2 ] );
         p_aux.ponPieza( piezas[ 1 ] );
         p_aux.ponPieza( piezas[ 0 ] );
         p_aux = new hnPoste();
         org.setPoste( 1, p_aux );
         p_aux = new hnPoste();
         org.setPoste( 2, p_aux );
         org.ultimo = null;

         ambientes[ 3 ] = new hnAmbiente( org, org.postes[2], 15 );

      // Caso 5
         org = new hnTablero( 3 );
         p_aux = new hnPoste();
         org.setPoste( 0, p_aux );
         p_aux.ponPieza( piezas[ 3 ] );
         p_aux.ponPieza( piezas[ 2 ] );
         p_aux = new hnPoste();
         org.setPoste( 1, p_aux );
         p_aux.ponPieza( piezas[ 4 ] );
         p_aux.ponPieza( piezas[ 1 ] );
         p_aux = new hnPoste();
         org.setPoste( 2, p_aux );
         p_aux.ponPieza( piezas[ 5 ] );
         p_aux.ponPieza( piezas[ 0 ] );
         org.ultimo = null;

         ambientes[ 4 ] = new hnAmbiente( org, org.postes[2], 18 );

      // Caso 6
         org = new hnTablero( 3 );
         p_aux = new hnPoste();
         org.setPoste( 0, p_aux );
         p_aux.ponPieza( piezas[ 5 ] );
         p_aux.ponPieza( piezas[ 4 ] );
         p_aux.ponPieza( piezas[ 3 ] );
         p_aux.ponPieza( piezas[ 2 ] );
         p_aux.ponPieza( piezas[ 1 ] );
         p_aux.ponPieza( piezas[ 0 ] );
         p_aux = new hnPoste();
         org.setPoste( 1, p_aux );
         p_aux = new hnPoste();
         org.setPoste( 2, p_aux );
         org.ultimo = null;

         ambientes[ 5 ] = new hnAmbiente( org, org.postes[2], 63 );
      }
   }
 
